@@ -1,6 +1,4 @@
-"""
-kombu.transport.qpid
-=======================
+"""Qpid Transport.
 
 `Qpid`_ transport using `qpid-python`_ as the client and `qpid-tools`_ for
 broker management.
@@ -25,9 +23,9 @@ or to install the requirements manually:
     to underlying dependencies not being compatible. This version is
     tested and works with with Python 2.7.
 
-.. _`Qpid`: http://qpid.apache.org/
-.. _`qpid-python`: http://pypi.python.org/pypi/qpid-python/
-.. _`qpid-tools`: http://pypi.python.org/pypi/qpid-tools/
+.. _`Qpid`: https://qpid.apache.org/
+.. _`qpid-python`: https://pypi.python.org/pypi/qpid-python/
+.. _`qpid-tools`: https://pypi.python.org/pypi/qpid-tools/
 
 Authentication
 ==============
@@ -77,16 +75,15 @@ The :attr:`~kombu.Connection.transport_options` argument to the
 options override and replace any other default or specified values. If using
 Celery, this can be accomplished by setting the
 *BROKER_TRANSPORT_OPTIONS* Celery option.
-
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
+from collections import OrderedDict
 import os
 import select
 import socket
 import ssl
 import sys
-import time
 import uuid
 
 from gettext import gettext as _
@@ -119,11 +116,10 @@ except ImportError:  # pragma: no cover
     qpid = None
 
 
-from kombu.five import Empty, items
+from kombu.five import Empty, items, monotonic, PY3
 from kombu.log import get_logger
 from kombu.transport.virtual import Base64, Message
 from kombu.transport import base
-from kombu.utils.compat import OrderedDict
 
 
 logger = get_logger(__name__)
@@ -134,12 +130,12 @@ OBJECT_ALREADY_EXISTS_STRING = 'object already exists'
 VERSION = (1, 0, 0)
 __version__ = '.'.join(map(str, VERSION))
 
-PY3 = sys.version_info[0] == 3
-
 
 def dependency_is_none(dependency):
-    """Return True if the dependency is None, otherwise False. This is done
-    using a function so that tests can mock this behavior easily.
+    """Return True if the dependency is None, otherwise False.
+
+    This is done using a function so that tests can mock this
+    behavior easily.
 
     :param dependency: The module to check if it is None
     :return: True if dependency is None otherwise False.
@@ -149,7 +145,7 @@ def dependency_is_none(dependency):
 
 
 class AuthenticationFailure(Exception):
-    pass
+    """Cannot authenticate with Qpid."""
 
 
 class QoS(object):
@@ -186,8 +182,7 @@ class QoS(object):
         self._not_yet_acked = OrderedDict()
 
     def can_consume(self):
-        """Return True if the :class:`~kombu.transport.qpid.Channel` can
-        consume more messages, else False.
+        """Return True if the :class:`Channel` can consume more messages.
 
         Used to ensure the client adheres to currently active prefetch
         limits.
@@ -204,8 +199,7 @@ class QoS(object):
         )
 
     def can_consume_max_estimate(self):
-        """Return the remaining message capacity for the associated
-        :class:`kombu.transport.qpid.Channel`.
+        """Return the remaining message capacity.
 
         Returns an estimated number of outstanding messages that a
         :class:`kombu.transport.qpid.Channel` can accept without
@@ -238,8 +232,9 @@ class QoS(object):
         self._not_yet_acked[delivery_tag] = message
 
     def get(self, delivery_tag):
-        """Get an un-ACKed message by delivery_tag. If called with an invalid
-        delivery_tag a :exc:`KeyError` is raised.
+        """Get an un-ACKed message by delivery_tag.
+
+        If called with an invalid delivery_tag a :exc:`KeyError` is raised.
 
         :param delivery_tag: The delivery tag associated with the message
             to be returned.
@@ -430,8 +425,9 @@ class Channel(base.StdChannel):
             rx.close()
         return message
 
-    def _put(self, routing_key, message, exchange=None, **kwargs):
-        """Synchronous send of a single message onto a queue or exchange.
+    def _put(self, routing_key, message, exchange=None, durable=True,
+             **kwargs):
+        """Synchronously send a single message onto a queue or exchange.
 
         An internal method which synchronously sends a single message onto
         a given queue or exchange. If exchange is not specified,
@@ -463,6 +459,9 @@ class Channel(base.StdChannel):
             should be sent on. If no exchange is specified, the message is
             sent directly to a queue specified by routing_key.
         :type exchange: str
+        :keyword durable: whether or not the message should persist or be
+            durable.
+        :type durable: bool
 
         """
         if not exchange:
@@ -475,6 +474,7 @@ class Channel(base.StdChannel):
             msg_subject = str(routing_key)
         sender = self.transport.session.sender(address)
         qpid_message = qpid.messaging.Message(content=message,
+                                              durable=durable,
                                               subject=msg_subject)
         try:
             sender.send(qpid_message, sync=True)
@@ -732,7 +732,7 @@ class Channel(base.StdChannel):
                 raise exc
 
     def exchange_delete(self, exchange_name, **kwargs):
-        """Delete an exchange specified by name
+        """Delete an exchange specified by name.
 
         :param exchange_name: The name of the exchange to be deleted.
         :type exchange_name: str
@@ -847,13 +847,13 @@ class Channel(base.StdChannel):
         try:
             qpid_message = self._get(queue)
             raw_message = qpid_message.content
-            message = self.Message(self, raw_message)
+            message = self.Message(raw_message, channel=self)
             self.transport.session.acknowledge(message=qpid_message)
             return message
         except Empty:
             pass
 
-    def basic_ack(self, delivery_tag):
+    def basic_ack(self, delivery_tag, multiple=False):
         """Acknowledge a message by delivery_tag.
 
         Acknowledges a message referenced by delivery_tag. Messages can
@@ -867,8 +867,12 @@ class Channel(base.StdChannel):
         :param delivery_tag: The delivery tag associated with the message
             to be acknowledged.
         :type delivery_tag: uuid.UUID
+        :param multiple: not implemented. If set to True an AssertionError
+            is raised.
+        :type multiple: bool
 
         """
+        assert multiple is False
         self.qos.ack(delivery_tag)
 
     def basic_reject(self, delivery_tag, requeue=False):
@@ -961,7 +965,7 @@ class Channel(base.StdChannel):
 
         def _callback(qpid_message):
             raw_message = qpid_message.content
-            message = self.Message(self, raw_message)
+            message = self.Message(raw_message, channel=self)
             delivery_tag = message.delivery_tag
             self.qos.append(qpid_message, delivery_tag)
             if no_ack:
@@ -1209,7 +1213,9 @@ class Channel(base.StdChannel):
 
 
 class Connection(object):
-    """Encapsulate a connection object for the
+    """Qpid Connection.
+
+    Encapsulate a connection object for the
     :class:`~kombu.transport.qpid.Transport`.
 
     :param host: The host that connections should connect to.
@@ -1331,7 +1337,7 @@ class Connection(object):
         return self._qpid_conn
 
     def close(self):
-        """Close the connection
+        """Close the connection.
 
         Closing the connection will close all associated session, senders, or
         receivers used by the Connection.
@@ -1428,9 +1434,7 @@ class Transport(base.Transport):
     def __init__(self, *args, **kwargs):
         self.verify_runtime_environment()
         super(Transport, self).__init__(*args, **kwargs)
-        self.r, self._w = os.pipe()
-        if fcntl is not None:
-            fcntl.fcntl(self.r, fcntl.F_SETFL, os.O_NONBLOCK)
+        self.use_async_interface = False
 
     def verify_runtime_environment(self):
         """Verify that the runtime environment is acceptable.
@@ -1471,10 +1475,12 @@ class Transport(base.Transport):
                 'qpid-python`.')
 
     def _qpid_message_ready_handler(self, session):
-        os.write(self._w, '0')
+        if self.use_async_interface:
+            os.write(self._w, '0')
 
     def _qpid_async_exception_notify_handler(self, obj_with_exception, exc):
-        os.write(self._w, 'e')
+        if self.use_async_interface:
+            os.write(self._w, 'e')
 
     def on_readable(self, connection, loop):
         """Handle any messages associated with this Transport.
@@ -1486,9 +1492,9 @@ class Transport(base.Transport):
         all available events are drained through a call to
         :meth:`drain_events`.
 
-        The behavior of self.r is adjusted in __init__ to be non-blocking,
-        ensuring that an accidental call to this method when no more messages
-        will arrive will not cause indefinite blocking.
+        The file descriptor self.r is modified to be non-blocking, ensuring
+        that an accidental call to this method when no more messages will
+        not cause indefinite blocking.
 
         Nothing is expected to be returned from :meth:`drain_events` because
         :meth:`drain_events` handles messages by calling callbacks that are
@@ -1522,7 +1528,7 @@ class Transport(base.Transport):
         :type connection: kombu.transport.qpid.Connection
         :param loop: The asynchronous loop object that contains epoll like
             functionality.
-        :type loop: kombu.async.Hub
+        :type loop: kombu.asynchronous.Hub
 
         """
         os.read(self.r, 1)
@@ -1554,9 +1560,13 @@ class Transport(base.Transport):
             this Transport.
         :type connection: kombu.transport.qpid.Connection
         :param loop: A reference to the external loop.
-        :type loop: kombu.async.hub.Hub
+        :type loop: kombu.asynchronous.hub.Hub
 
         """
+        self.r, self._w = os.pipe()
+        if fcntl is not None:
+            fcntl.fcntl(self.r, fcntl.F_SETFL, os.O_NONBLOCK)
+        self.use_async_interface = True
         loop.add_reader(self.r, self.on_readable, connection, loop)
 
     def establish_connection(self):
@@ -1676,7 +1686,7 @@ class Transport(base.Transport):
         :type timeout: int
 
         """
-        start_time = time.time()
+        start_time = monotonic()
         elapsed_time = -1
         while elapsed_time < timeout:
             try:
@@ -1687,7 +1697,7 @@ class Transport(base.Transport):
                 raise socket.timeout()
             else:
                 connection._callbacks[queue](message)
-            elapsed_time = time.time() - start_time
+            elapsed_time = monotonic() - start_time
         raise socket.timeout()
 
     def create_channel(self, connection):
@@ -1727,9 +1737,10 @@ class Transport(base.Transport):
 
     def __del__(self):
         """Ensure file descriptors opened in __init__() are closed."""
-        for fd in (self.r, self._w):
-            try:
-                os.close(fd)
-            except OSError:
-                # ignored
-                pass
+        if getattr(self, 'use_async_interface', False):
+            for fd in (self.r, self._w):
+                try:
+                    os.close(fd)
+                except OSError:
+                    # ignored
+                    pass
